<?php

namespace Litematrimony\Preference\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litematrimony\Preference\Events\PreferenceWorkflow as PreferenceWorkflowEvent;
use Litematrimony\Preference\Notifications\PreferenceWorkflow as PreferenceWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class PreferenceWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(PreferenceWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $preference is submitted.
     *
     * @param PreferenceWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(PreferenceWorkflowEvent $event)
    {
        $client = $event->$preference->client;
        Notification::send($client, new PreferenceWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $preference is published.
     *
     * @param PreferenceWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(PreferenceWorkflowEvent $event)
    {
        $client = $event->$preference->client;
        Notification::send($client, new PreferenceWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $preference is approved.
     *
     * @param PreferenceWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(PreferenceWorkflowEvent $event)
    {
        $client = $event->$preference->client;
        Notification::send($client, new PreferenceWorkflowNotification($event));
    }

    /**
     * Handles the $preference workflow event as a listener.
     *
     * @param PreferenceWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(PreferenceWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
